@@ -2,7 +2,7 @@ package Test2::API::Instance;
 use strict;
 use warnings;
 
-our $VERSION = '1.302113';
+our $VERSION = '1.302118';
 
 
 our @CARP_NOT = qw/Test2::API Test2::API::Instance Test2::IPC::Driver Test2::Formatter/;
@@ -36,6 +36,7 @@ use Test2::Util::HashBase qw{
     context_acquire_callbacks
     context_init_callbacks
     context_release_callbacks
+    pre_subtest_callbacks
 };
 
 sub DEFAULT_IPC_TIMEOUT() { 30 }
@@ -135,6 +136,7 @@ sub reset {
     $self->{+CONTEXT_ACQUIRE_CALLBACKS} = [];
     $self->{+CONTEXT_INIT_CALLBACKS}    = [];
     $self->{+CONTEXT_RELEASE_CALLBACKS} = [];
+    $self->{+PRE_SUBTEST_CALLBACKS}     = [];
 
     $self->{+STACK} = Test2::API::Stack->new;
 }
@@ -274,6 +276,18 @@ sub add_post_load_callback {
     $code->() if $self->{+LOADED};
 }
 
+sub add_pre_subtest_callback {
+    my $self =  shift;
+    my ($code) = @_;
+
+    my $rtype = reftype($code) || "";
+
+    confess "Pre-subtest callbacks must be coderefs"
+        unless $code && $rtype eq 'CODE';
+
+    push @{$self->{+PRE_SUBTEST_CALLBACKS}} => $code;
+}
+
 sub load {
     my $self = shift;
     unless ($self->{+LOADED}) {
@@ -441,8 +455,10 @@ sub _ipc_wait {
                 last if $pid == -1;
                 next unless $err;
                 $fail++;
-                $err = $err >> 8;
-                warn "Process $pid did not exit cleanly (status: $err)\n";
+
+                my $sig = $err & 127;
+                my $exit = $err >> 8;
+                warn "Process $pid did not exit cleanly (wstat: $err, exit: $exit, sig: $sig)\n";
             }
 
             alarm 0;
@@ -676,6 +692,10 @@ Get all context init callbacks.
 
 Get all context release callbacks.
 
+=item $arrayref = $obj->pre_subtest_callbacks
+
+Get all pre-subtest callbacks.
+
 =item $obj->add_context_init_callback(sub { ... })
 
 Add a context init callback. Subs are called every time a context is created. Subs
@@ -687,6 +707,12 @@ Add a context release callback. Subs are called every time a context is released
 get the released context as their only argument. These callbacks should not
 call release on the context.
 
+=item $obj->add_pre_subtest_callback(sub { ... })
+
+Add a pre-subtest callback. Subs are called every time a subtest is
+going to be run. Subs get the subtest name, coderef, and any
+arguments.
+
 =item $obj->set_exit()
 
 This is intended to be called in an C<END { ... }> block. This will look at
